<?php

namespace Bixo\Salary\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Salary\Http\Requests\AdjustmentsRequest;
use Bixo\Salary\Interfaces\AdjustmentsRepositoryInterface;
use Bixo\Salary\Models\Adjustments;

/**
 * Resource controller class for adjustments.
 */
class AdjustmentsResourceController extends BaseController
{

    /**
     * Initialize adjustments resource controller.
     *
     * @param type AdjustmentsRepositoryInterface $adjustments
     *
     * @return null
     */
    public function __construct(AdjustmentsRepositoryInterface $adjustments)
    {
        parent::__construct();
        $this->repository = $adjustments;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Salary\Repositories\Criteria\AdjustmentsResourceCriteria::class);
    }

    /**
     * Display a list of adjustments.
     *
     * @return Response
     */
    public function index(AdjustmentsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Salary\Repositories\Presenter\AdjustmentsPresenter::class)
                ->$function();
        }

        $adjustments = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('salary::adjustments.names'))
            ->view('salary::adjustments.index', true)
            ->data(compact('adjustments', 'view'))
            ->output();
    }

    /**
     * Display adjustments.
     *
     * @param Request $request
     * @param Model   $adjustments
     *
     * @return Response
     */
    public function show(AdjustmentsRequest $request, Adjustments $adjustments)
    {

        if ($adjustments->exists) {
            $view = 'salary::adjustments.show';
        } else {
            $view = 'salary::adjustments.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('salary::adjustments.name'))
            ->data(compact('adjustments'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new adjustments.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AdjustmentsRequest $request)
    {

        $adjustments = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('salary::adjustments.name')) 
            ->view('salary::adjustments.create', true) 
            ->data(compact('adjustments'))
            ->output();
    }

    /**
     * Create new adjustments.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AdjustmentsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $adjustments                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('salary::adjustments.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('salary/adjustments/' . $adjustments->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/salary/adjustments'))
                ->redirect();
        }

    }

    /**
     * Show adjustments for editing.
     *
     * @param Request $request
     * @param Model   $adjustments
     *
     * @return Response
     */
    public function edit(AdjustmentsRequest $request, Adjustments $adjustments)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('salary::adjustments.name'))
            ->view('salary::adjustments.edit', true)
            ->data(compact('adjustments'))
            ->output();
    }

    /**
     * Update the adjustments.
     *
     * @param Request $request
     * @param Model   $adjustments
     *
     * @return Response
     */
    public function update(AdjustmentsRequest $request, Adjustments $adjustments)
    {
        try {
            $attributes = $request->all();

            $adjustments->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('salary::adjustments.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('salary/adjustments/' . $adjustments->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('salary/adjustments/' . $adjustments->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the adjustments.
     *
     * @param Model   $adjustments
     *
     * @return Response
     */
    public function destroy(AdjustmentsRequest $request, Adjustments $adjustments)
    {
        try {

            $adjustments->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('salary::adjustments.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('salary/adjustments/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('salary/adjustments/' . $adjustments->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple adjustments.
     *
     * @param Model   $adjustments
     *
     * @return Response
     */
    public function delete(AdjustmentsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('salary::adjustments.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('salary/adjustments'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/salary/adjustments'))
                ->redirect();
        }

    }

    /**
     * Restore deleted adjustments.
     *
     * @param Model   $adjustments
     *
     * @return Response
     */
    public function restore(AdjustmentsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('salary::adjustments.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/salary/adjustments'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/salary/adjustments/'))
                ->redirect();
        }

    }

}
